<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2004 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: StatisticsCommon.php 1122 2004-12-09 15:45:11Z andrew $
*/

/**
 * @package    MaxMaintenance
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/libraries/pear/Date.php';

/**
 * An abstract class that defines the interface and some common methods for
 * summarising raw data into statistics, for all Max modules.
 *
 * @abstract
 */
class Maintenance_Statistics_Common
{
    var $module;
    var $now;    
    var $conf;
    var $dal;
    var $report;
    var $updateUsingOI;
    var $lastDateOI;
    var $updateOI;
    var $upToDateOI;
    var $lastDateHourly;
    var $updateHourly;    
    var $upToDateHourly;
    
    /**
     * A common constructor method that can be used for all children of the class.
     * Must have the class $module variable set before being called in the child
     * constructor method.
     *
     * @param Date $now Optional Date representing the current date/time. Normally
     *                  just used for testing the class constructor, or for 
     *                  faking the current time in order to re-generate past statistics,
     *                  in the case of faulty raw data tables.
     */
    function Maintenance_Statistics_Common($now = null)
    {
        if (is_null($now)) {
            $this->now = new Date();
        } else {
            $this->now = $now;
        }        
        $this->conf =& $GLOBALS['_MAX']['CONF'];
        Dal_Statistics::instance($this->module, $this->dal);
        $this->report  = "Maintenance Statistics Report: $this->module\n";
        debug("Running Maintenance Statistics for $this->module", PEAR_LOG_INFO);
        $this->report .= "=====================================\n\n";
        $this->report .= 'Current time is ' . $this->now->format('%Y-%m-%d %H:%M:%S') . ".\n";
        debug('Current time is ' . $this->now->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
        // Which of the operation interval and an hour is smaller?
        if ($this->conf['maintenance']['operation_interval'] <= 60) {
            $this->updateUsingOI = true;
        } else {
            $this->updateUsingOI = false;
        }
        // Don't update unless the time is right!
        $this->updateOI     = false;
        $this->updateHourly = false;
        // Determine when the last operation interval update happened
        $this->lastDateOI = $this->dal->getMaintenanceStatisticsLastRunInfo(0, $now);
        if (is_null($this->lastDateOI)) {
            $this->report .= "Maintenance statistics has never been run before, and there is no raw data\n" .
                             "in the database, so maintenance statistics will not be run for the operation interval.\n\n";
            $message = "Maintenance statistics has never been run before, and there is no raw data " .
                       "in the database, so maintenance statistics will not be run for the operation interval";
            debug($message, PEAR_LOG_DEBUG);
        } else {
            $this->report .= "Maintenance statistics last updated OI statistics to " . 
                             $this->lastDateOI->format('%Y-%m-%d %H:%M:%S') . ".\n";
            $message = "Maintenance statistics last updated OI statistics to " . 
                       $this->lastDateOI->format('%Y-%m-%d %H:%M:%S') . ".";
            debug($message, PEAR_LOG_DEBUG);
            $requiredDate = $this->lastDateOI;
            $requiredDate->addSeconds($this->conf['maintenance']['operation_interval'] * 60);
            $this->report .= 'Current time must be after ' . $requiredDate->format('%Y-%m-%d %H:%M:%S') . 
                             " for the next operation interval update to happen.\n";
            $message = 'Current time must be after ' . $requiredDate->format('%Y-%m-%d %H:%M:%S') . 
                       ' for the next operation interval update to happen';
            debug($message, PEAR_LOG_DEBUG);
            if (Date::compare($this->now, $requiredDate) > 0) {
                $this->updateOI = true;
                // Update OI stats to the end of the previous (not current) operation interval
                list($this->upToDateOI, $_) = Maintenance::convertDateToOperationIntervalStartAndEndDates($this->now);
                $this->upToDateOI->subtractSeconds(1);
            } else {
                // An operation interval hasn't passed, so don't update
                $this->report .= "At least {$this->conf['maintenance']['operation_interval']} minutes have " .
                                 "not passed since the last operation interval update.\n";
                $message = "At least {$this->conf['maintenance']['operation_interval']} minutes have " .
                           'not passed since the last operation interval update';
                debug($message, PEAR_LOG_DEBUG);
            }
        }
        // Determine when the last hourly update happened
        $this->lastDateHourly = $this->dal->getMaintenanceStatisticsLastRunInfo(1, $now);
        if (is_null($this->lastDateHourly)) {
            // There are no statistics, cannot run
            $this->report .= "Maintenance statistics has never been run before, and there is no raw data\n" .
                             "in the database, so maintenance statistics will not be run for the hour.\n\n";
            $message = "Maintenance statistics has never been run before, and there is no raw data " .
                       "in the database, so maintenance statistics will not be run for the hour";
            debug($message, PEAR_LOG_DEBUG);
        } else {
            $this->report .= 'Maintenance statistics last updated hourly statistics to ' .
                             $this->lastDateHourly->format('%Y-%m-%d %H:%M:%S') . ".\n";
            $message = 'Maintenance statistics last updated hourly statistics to ' .
                       $this->lastDateHourly->format('%Y-%m-%d %H:%M:%S') . ".";
            debug($message, PEAR_LOG_DEBUG);
            $requiredDate = $this->lastDateHourly;
            $requiredDate->addSeconds(60 * 60);
            $this->report .= 'Current time must be after ' . $requiredDate->format('%Y-%m-%d %H:%M:%S') .
                             " for the next hourly update to happen.\n";
            $message = 'Current time must be after ' . $requiredDate->format('%Y-%m-%d %H:%M:%S') .
                       ' for the next hourly update to happen';
            debug($message, PEAR_LOG_DEBUG);
            if (Date::compare($this->now, $requiredDate) > 0) {
                $this->updateHourly = true;
                // Update hourly stats to the end of the previous (not current) hour
                $this->upToDateHourly = new Date($this->now->format('%Y-%m-%d %H:00:00'));
                $this->upToDateHourly->subtractSeconds(1);
            } else {
                // An hour hasn't passed, so don't update
                $this->report .= "At least 60 minutes have NOT passed since the last hourly update.\n";
                $message = 'At least 60 minutes have NOT passed since the last hourly update';
                debug($message, PEAR_LOG_DEBUG);
            }
        }
        if ($this->updateOI || $this->updateHourly) {
            $this->report .= "Maintenance statistics will be run.\n\n";
            $message = "Maintenance statistics will be run";
            debug($message, PEAR_LOG_DEBUG);
            if ($this->updateOI) {
                $this->report .= "The statistics will be updated on the basis of operation interval.\n\n";
                $message = 'The statistics will be updated on the basis of operation interval';
                debug($message, PEAR_LOG_DEBUG);
            }
            if ($this->updateHourly) {
                $this->report .= "The statistics will be updated on the basis of the hour.\n\n";
                $message = 'The statistics will be updated on the basis of the hour';
                debug($message, PEAR_LOG_DEBUG);
            }
        } else {
            $this->report .= "Maintenance statistics will NOT be run.\n\n";   
            $message = 'Maintenance statistics will NOT be run';
            debug($message, PEAR_LOG_DEBUG);
        }
    }
    
    /**
     * A method to summarise raw data into the intermediate statistics tables.
     */
    function summariseIntermediate() {}
    
    /**
     * A method to summarise intermediate statistics into summary (final) statistics
     * tables, where required.
     */
    function summariseFinal() {}

    /**
     * A method to delete raw data that is no-longer required, if the option to
     * do so is set in the configuration .ini file.
     */
    function deleteOldData()
    {
        if ($this->conf['maintenance']['compact_stats']) {
            $time = time();            
            if (($this->updateHourly) && ($this->updateUsingOI)) {
                // Have updated the hourly stats, and the operation interval is less than or equal to an
                // hour, so it's safe to delete the statistics that have been summarised now
                $summarisedTo = $this->upToDateHourly;
            } else if (($this->updateOI) && (!$this->updateUsingOI)) {
                // Have updated the operation interval stats, and the operation interval is more than
                // an hour, so it's safe to delete the statistics that have been summarised now            
                $summarisedTo = $this->upToDateOI;
            }
            if (!is_null($summarisedTo)) {
                $rows = $this->dal->deleteOldData($summarisedTo);
                $time = time() - $time;
                if ($this->conf['table']['split']) {
                    $this->report .= "Dropped $rows tables of old, summarised data in $time seconds.\n";
                    $message = "Dropped $rows tables of old, summarised data in $time seconds";
                } else {
                    $this->report .= "Deleted $rows rows of old, summarised data in $time seconds.\n";
                    $message = "Deleted $rows rows of old, summarised data in $time seconds";
                }
                debug($message, PEAR_LOG_DEBUG);
            }
        }
    }

    /**
     * A method to log the completion of summarising the raw data into statistics.
     *
     * @param Date $now Optional Date representing the current date/time. Normally
     *                  just used for testing the class constructor.
     */
    function logCompletion($now = null)
    {
        if (is_null($now)) {
            $end = new Date();
        } else {
            $end = $now;
        }
        if (($this->updateHourly) && ($this->updateOI)) {
            $this->dal->logCompletion($this->now, $end, 2, $this->upToDateOI, $this->report);
        } else if ($this->updateHourly) {
            $this->dal->logCompletion($this->now, $end, 1, $this->upToDateHourly, $this->report);
        } else if ($this->updateOI) {
            $this->dal->logCompletion($this->now, $end, 0, $this->upToDateOI, $this->report);
        }
    }
}

?>